#!/usr/bin/env python

import sys
import emod_api.interventions.common as comm
import emodpy_malaria.interventions.common as malcomm
from emodpy_hiv.interventions.peek_camp import encode
import emod_api.campaign as camp

schema_path = sys.argv[2]
camp.set_schema( schema_path )
mycamp = encode( sys.argv[1] )

def iv_class_factory_ish( iv_name, payload ):
    iv = None
    try:
        method_to_call = getattr(comm, iv_name)
    except Exception as ex:
        try:
            method_to_call = getattr(malcomm, iv_name)
        except Exception as ex:
            method_to_call = None
    if iv_name == "PropertyValueChanger":
        ips = payload.split( ":" )
        iv = method_to_call( camp, Target_Property_Key=ips[0], Target_Property_Value=ips[1] )
    elif iv_name == "AntiretroviralTherapy":
        from emodpy_hiv.interventions.art import new_intervention as art
        iv = art( camp )
    elif iv_name == "BroadcastEvent":
        iv = method_to_call( camp, Event_Trigger=payload )
    elif iv_name == "HIVRandomChoice":
        from emodpy_hiv.interventions.random import new_diagnostic as hrc
        import json
        payload = json.loads( payload.replace( "'", '"' ) )
        iv = hrc( camp, choices=payload )
    elif iv_name == "ModifyStiCoInfectionStatus":
        from emodpy_hiv.interventions.modcoinf import new_intervention as coinf
        iv = coinf( camp )
    elif iv_name == "HIVRapidHIVDiagnostic":
        from emodpy_hiv.interventions.rapiddiag import new_diagnostic as hrc
        bcast_signals = payload.split("/")
        iv = hrc( camp, Positive_Event=bcast_signals[0], Negative_Event=bcast_signals[1] )
    elif iv_name.startswith( "Delay" ):
        print( "Extract delay distro and param(s)." )
        # I don't think delays are passed in as interventions anymore, but as a delay param
        import pdb
        pdb.set_trace()
        iv = method_to_call( camp, Event_Trigger=payload )
    elif iv_name.startswith( "Outbreak" ):
        import emodpy_hiv.interventions.outbreak as ob
        iv = ob._Outbreak( camp )
    elif iv_name == "MigrateIndividuals":
        from emod_api.interventions.migration import _new_migration_intervention
        iv = _new_migration_intervention( camp, nodeid_to_migrate_to=int(payload) ) # defaults won't work
        print( "NOTE: MigrateIndividuals will need more params for durations!" )
    else:
        if method_to_call is None:
            print( f"ERROR: No function found to create intervention {iv_name}." )
            return None
        try:
            iv = method_to_call( camp )
        except Exception as ex:
            print( f"Exception invoking method for iv_name {iv_name}." )
            print( str(ex) )
    return iv

def convert_delay( delay ):
    """
    Param comes in as, e.g., FIXED/30, and needs to be converted into, e.g., { "Distribution": "FIXED_DURATION", "Delayed_Period": 30 }
    """
    if delay is None:
        return None
    delay_dict_return = {}
    delay_parsed = delay.split( "/" )
    delay_dict_return[ "Delay_Distribution"] = f"{delay_parsed[0]}_DURATION"
    if delay.startswith( "FIXED" ):
        delay_dict_return["Delay_Period"]  = float(delay_parsed[1])
    return delay_dict_return 

for enc_event in mycamp:
    # what
    iv_name = enc_event["iv_name"]
    payload = enc_event["payload"] if "payload" in enc_event else None
    if type(iv_name) is list:
        iv = list()
        for idx in range(len(iv_name)):
            act_iv_name = iv_name[idx]
            act_payload = payload[idx]
            intervention = iv_class_factory_ish( act_iv_name, act_payload )
            iv.append( intervention )
    else:
        iv = [iv_class_factory_ish( iv_name, payload )]

    # where
    nodes = enc_event["nodes"]

    # who
    coverage = enc_event["frac"]
    sex = enc_event["sex"] if "sex" in enc_event else "All"
    min_age = enc_event["min_age"] if "min_age" in enc_event else None
    max_age = enc_event["max_age"] if "max_age" in enc_event else None
    ips = enc_event["ips"] if "ips" in enc_event else None

    if "signal" in enc_event and enc_event["signal"]:
        signal = enc_event["signal"]
        # print( f"signal={signal}." )
        delay = enc_event["delay"] if "delay" in enc_event else None
        delay = convert_delay( delay )
        if delay:
            from emod_api.interventions.common import DelayedIntervention
            iv = [DelayedIntervention( camp, Configs=iv, Delay_Dict = delay )]
        event = comm.TriggeredCampaignEvent( camp, Start_Day=enc_event["start_day"], Node_Ids=nodes, Triggers=[signal], Event_Name="TCE", Intervention_List=iv, Demographic_Coverage=coverage, Property_Restrictions=ips, Target_Gender=sex )
        #event = comm.TriggeredCampaignEvent( camp, Start_Day=enc_event["start_day"], Node_Ids=nodes, Triggers=[signal], Event_Name="TCE", Intervention_List=iv, Demographic_Coverage=coverage, Target_Age_Min=min_age, Target_Age_Max=max_age )
        camp.add( event )
    else:
        if "reps" in enc_event:
            reps = int(enc_event[ "reps" ])
            gap = float(enc_event[ "gap" ])
        else:
            reps = 1
            gap = -1
        try:
            event = comm.ScheduledCampaignEvent( camp, Start_Day=enc_event["start_day"], Node_Ids=nodes, Intervention_List=iv, Number_Repetitions=reps, Timesteps_Between_Repetitions=gap, Demographic_Coverage=coverage, Property_Restrictions=ips, Target_Gender=sex )
        #event = comm.ScheduledCampaignEvent( camp, Start_Day=enc_event["start_day"], Node_Ids=nodes, Intervention_List=iv, Number_Repetitions=reps, Timesteps_Between_Repetitions=gap, Demographic_Coverage=coverage, Target_Age_Min=min_age, Target_Age_Max=max_age )
        except Exception as ex:
            print( str( ex ) )
            import pdb
            pdb.set_trace()
        camp.add( event )

outfile = "test_campaign.json"
print( f"\n\nWriting {outfile} to disk." )
camp.save( outfile )
